@@ -54,7 +54,7 @@ pub enum CertificationError {
     TargetNotLive(Certification, SystemTime, #[source] anyhow::Error),
 
     #[error("{0}: target certificate is not valid \
-             as of the reference time ({})",
+             as of the certification time ({})",
             format_time(.1))]
     TargetNotValid(Certification, SystemTime, #[source] anyhow::Error),
 
@@ -512,20 +512,9 @@ impl Certification {
                     err).into());
             }
 
-            if let Some(ref ua) = ua {
-                certification
-                    .clone()
-                    .verify_userid_binding(
-                        possible_issuer.primary_key().key(),
-                        target.primary_key().key(),
-                        ua.userid())?;
-            } else {
-                certification
-                    .clone()
-                    .verify_direct_key(
-                        possible_issuer.primary_key().key(),
-                        target.primary_key().key())?;
-            }
+            certification
+                .clone()
+                .verify_signature(possible_issuer.primary_key().key())?;
 
             // Ignore if the issuer is not alive at the
             // certification time (not the reference time!).
@@ -630,10 +619,7 @@ impl Certification {
                     }) {
                         if let Ok(()) = rev
                             .clone()
-                            .verify_userid_revocation(
-                                possible_issuer.primary_key().key(),
-                                target.primary_key().key(),
-                                ua.userid())
+                            .verify_signature(possible_issuer.primary_key().key())
                         {
                             t!("issuer revoked certification, ignoring");
                             return Err(
@@ -724,7 +710,7 @@ impl Certification {
                    err);
                 return Err(CertificationError::TargetNotValid(
                     (possible_issuer, target, certification).into(),
-                    reference_time, err).into());
+                    certification_time, err).into());
             }
         };
 
@@ -1159,12 +1145,21 @@ impl CertificationSet {
         }
     }
 
-    /// Returns an iterator over all of the certifications.
+    /// Returns an iterator over all of the user ids, and their
+    /// certifications.
     pub fn certifications(&self)
         -> impl Iterator<Item=(Option<&UserID>, &[Certification])>
     {
         self.certifications.iter().map(|(userid, c)| (userid.as_ref(), &c[..]))
     }
+
+    /// Returns an iterator over the certifications.
+    pub fn into_certifications(self)
+        -> impl Iterator<Item=Certification>
+    {
+        self.certifications.into_iter()
+            .flat_map(|(_userid, c)| c.into_iter())
+    }
 }
 
 #[cfg(test)]
