@@ -1,4 +1,4 @@
-use std::collections::HashMap;
+use std::collections::BTreeMap;
 use std::io::Write;
 use std::process::Command;
 use std::process::Stdio;
@@ -71,7 +71,7 @@ pub enum OwnerTrust {
     Other(usize),
 }
 
-pub fn export_ownertrust() -> Result<HashMap<Fingerprint, OwnerTrust>>
+pub fn export_ownertrust() -> Result<BTreeMap<Fingerprint, OwnerTrust>>
 {
     tracer!(TRACE, "export_ownertrust");
 
@@ -100,7 +100,7 @@ pub fn export_ownertrust() -> Result<HashMap<Fingerprint, OwnerTrust>>
     // should work.  If not, abort.
     let output = String::from_utf8(result.stdout)?;
 
-    let mut db = HashMap::new();
+    let mut db = BTreeMap::new();
 
     for line in output.split('\n') {
         if line == "" {
