@@ -1,6 +1,6 @@
 use std::borrow::Borrow;
-use std::collections::hash_map::Entry;
-use std::collections::HashMap;
+use std::collections::btree_map::Entry;
+use std::collections::BTreeMap;
 
 use sequoia_openpgp as openpgp;
 use openpgp::Fingerprint;
@@ -109,7 +109,7 @@ impl<'a> ChainFilter<'a> {
 pub struct SuppressCertificationFilter {
     // A certification's trust amount will be suppressed by this
     // amount.
-    amount: HashMap<(Fingerprint, Fingerprint), usize>,
+    amount: BTreeMap<(Fingerprint, Fingerprint), usize>,
 }
 
 impl CertificationFilter for SuppressCertificationFilter {
@@ -139,7 +139,7 @@ impl SuppressCertificationFilter {
     /// Returns a new, empty `SuppressCertificationFilter`.
     pub fn new() -> Self {
         Self {
-            amount: HashMap::new(),
+            amount: BTreeMap::new(),
         }
     }
 
@@ -174,7 +174,7 @@ impl SuppressCertificationFilter {
 pub struct SuppressIssuerFilter {
     // A certification's trust amount will be suppressed by this
     // amount.
-    amount: HashMap<Fingerprint, usize>,
+    amount: BTreeMap<Fingerprint, usize>,
 }
 
 impl CertificationFilter for SuppressIssuerFilter {
@@ -202,7 +202,7 @@ impl SuppressIssuerFilter {
     /// Returns a new, empty `SuppressIssuerFilter`.
     pub fn new() -> Self {
         Self {
-            amount: HashMap::new(),
+            amount: BTreeMap::new(),
         }
     }
 
@@ -237,7 +237,7 @@ impl SuppressIssuerFilter {
 /// A filter that caps (limits) the capacity of some certificates.
 pub struct CapCertificateFilter {
     // A certificate's trust amount will be limited to this amount.
-    cap: HashMap<Fingerprint, usize>,
+    cap: BTreeMap<Fingerprint, usize>,
 }
 
 impl CertificationFilter for CapCertificateFilter {
@@ -265,7 +265,7 @@ impl CapCertificateFilter {
     /// Returns a new, empty `CapCertificateFilter`.
     pub fn new() -> Self {
         Self {
-            cap: HashMap::new(),
+            cap: BTreeMap::new(),
         }
     }
 
