@@ -1,4 +1,5 @@
 use std::collections::BTreeSet;
+use std::collections::BTreeMap;
 use std::collections::HashMap;
 use std::io::Write;
 use std::time::SystemTime;
@@ -297,7 +298,7 @@ pub struct OutputNetwork {
     gossip: bool,
     certification_network: bool,
     certs: Vec<OutputCert>,
-    paths: HashMap<Fingerprint, Vec<OutputPath>>,
+    paths: BTreeMap<Fingerprint, Vec<OutputPath>>,
 }
 
 impl OutputNetwork {
@@ -307,7 +308,7 @@ impl OutputNetwork {
         certification_network: bool,
     ) -> Self {
         let certs = Vec::new();
-        let paths = HashMap::new();
+        let paths = BTreeMap::new();
         OutputNetwork {
             required_amount,
             gossip,
