@@ -1,7 +1,7 @@
 #[macro_use] mod log;
 
-use std::collections::HashMap;
-use std::collections::hash_map::Entry;
+use std::collections::BTreeMap;
+use std::collections::btree_map::Entry;
 use std::time::SystemTime;
 
 use anyhow::Context;
@@ -162,10 +162,10 @@ fn have_self_signed_userid(cert: &wot::CertSynopsis,
     }
 }
 
-/// Authenticate bindings defined by a Query on a Network
+/// Authenticate bindings on a Network.
 fn authenticate<S>(
     cli: &cli::Cli,
-    q: &wot::Query<'_, S>,
+    n: &wot::Network<S>,
     gossip: bool,
     userid: Option<&UserID>,
     certificate: Option<&KeyHandle>,
@@ -178,7 +178,7 @@ fn authenticate<S>(
         Some(match kh {
             KeyHandle::Fingerprint(fpr) => fpr.clone(),
             kh @ KeyHandle::KeyID(_) => {
-                let certs = q.network().lookup_synopses(kh)?;
+                let certs = n.lookup_synopses(kh)?;
                 if certs.is_empty() {
                     return Err(StoreError::NotFound(kh.clone()).into());
                 }
@@ -229,12 +229,12 @@ fn authenticate<S>(
         // and select the bindings where the User ID matches the email
         // address.
         bindings = if let Some(fingerprint) = fingerprint.as_ref() {
-            q.network().certified_userids_of(fingerprint)
+            n.certified_userids_of(fingerprint)
                 .into_iter()
                 .map(|userid| (fingerprint.clone(), userid))
                 .collect::<Vec<_>>()
         } else {
-            q.network().lookup_synopses_by_email(&email)
+            n.lookup_synopses_by_email(&email)
         };
 
         let email_normalized = userid_check.email_normalized()
@@ -258,7 +258,7 @@ fn authenticate<S>(
             bindings.push((fingerprint, userid.clone()));
         } else {
             // Fingerprint, no User ID.
-            bindings = q.network().certified_userids_of(&fingerprint)
+            bindings = n.certified_userids_of(&fingerprint)
                 .into_iter()
                 .map(|userid| (fingerprint.clone(), userid))
                 .collect();
@@ -266,7 +266,7 @@ fn authenticate<S>(
     } else if let Some(userid) = userid {
         // The caller did not specify a certificate.  Find all
         // bindings with the User ID.
-        bindings = q.network().lookup_synopses_by_userid(userid.clone())
+        bindings = n.lookup_synopses_by_userid(userid.clone())
             .into_iter()
             .map(|fpr| (fpr, userid.clone()))
             .collect();
@@ -274,7 +274,7 @@ fn authenticate<S>(
         // No User ID, no Fingerprint.
         // List everything.
 
-        bindings = q.network().certified_userids();
+        bindings = n.certified_userids();
 
         if let cli::Subcommand::List { pattern: Some(pattern), .. } = &cli.subcommand {
             // Or rather, just User IDs that match the pattern.
@@ -326,7 +326,7 @@ fn authenticate<S>(
         cli::OutputFormat::Dot => {
             Box::new(output::DotOutputNetwork::new(
                 required_amount,
-                q.roots(),
+                n.roots(),
                 gossip,
                 cli.certification_network,
             ))
@@ -344,7 +344,7 @@ fn authenticate<S>(
 
         let paths = if gossip {
             // Gossip.
-            let paths = q.gossip(
+            let paths = n.gossip(
                 fingerprint.clone(), userid.clone());
 
             // Sort so the shortest paths come first.
@@ -365,7 +365,7 @@ fn authenticate<S>(
                 .map(|p| (p, 0))
                 .collect::<Vec<(wot::Path, usize)>>()
         } else {
-            let paths = q.authenticate(
+            let paths = n.authenticate(
                 userid.clone(), fingerprint.clone(), required_amount);
 
             aggregated_amount = paths.amount();
@@ -390,7 +390,7 @@ fn authenticate<S>(
     if lint_input {
         // See if the target certificate exists.
         if let Some(kh) = certificate {
-            match q.network().lookup_synopses(kh) {
+            match n.lookup_synopses(kh) {
                 Err(err) => {
                     eprintln!("Looking up target certificate ({}): {}",
                              kh, err);
@@ -416,7 +416,7 @@ fn authenticate<S>(
 
                         // Check if the certificate has expired.
                         if let Some(e) = cert.expiration_time() {
-                            if e <= q.network().reference_time() {
+                            if e <= n.reference_time() {
                                 eprintln!("Warning: {} is expired.", kh);
                             }
                         }
@@ -432,14 +432,11 @@ fn authenticate<S>(
 
                         // See if there are any certifications made on
                         // this certificate.
-                        if let Ok(cs) = q.network()
-                            .certifications_of(&fpr, 0.into())
-                        {
-                            if cs.iter().all(|cs| {
-                                cs.certifications()
-                                    .all(|(_userid, certifications)| {
-                                        certifications.is_empty()
-                                    })
+                        if let Ok(css) = n.certifications_of(&fpr, 0.into()) {
+                            if css.iter().all(|cs| {
+                                cs.certifications().all(|(_userid, certifications)| {
+                                    certifications.is_empty()
+                                })
                             })
                             {
                                 eprintln!("Warning: {} has no valid \
@@ -473,9 +470,9 @@ fn authenticate<S>(
 
         // See if the trust roots exist.
         if ! gossip {
-            if q.roots().iter().all(|r| {
+            if n.roots().iter().all(|r| {
                 let fpr = r.fingerprint();
-                if let Err(err) = q.network().lookup_synopsis_by_fpr(&fpr) {
+                if let Err(err) = n.lookup_synopsis_by_fpr(&fpr) {
                     eprintln!("Looking up trust root ({}): {}.",
                              fpr, err);
                     true
@@ -502,8 +499,8 @@ fn authenticate<S>(
 }
 
 // For `sq-wot path`.
-fn check_path<'a: 'b, 'b, S>(cli: &cli::Cli, q: &wot::Query<'b, S>,
-                             policy: &dyn Policy)
+fn check_path<'a, S>(cli: &cli::Cli, q: &wot::Network<S>,
+                     policy: &dyn Policy)
     -> Result<()>
 where S: wot::store::Store + wot::store::Backend<'a>
 {
@@ -703,7 +700,7 @@ alone or combined with the other options.");
     }
 
     // Map any Key IDs to Fingerprints.
-    let mut keyid_index: Option<HashMap<KeyID, Vec<Fingerprint>>> = None;
+    let mut keyid_index: Option<BTreeMap<KeyID, Vec<Fingerprint>>> = None;
     let mut trust_roots: Vec<wot::Root> = trust_roots
         .into_iter()
         .filter_map(|(kh, amount)| {
@@ -716,8 +713,8 @@ alone or combined with the other options.");
                         Some(i) => i,
                         None => {
                             // Initialize the hash map lazily.
-                            let mut i: HashMap<KeyID, Vec<Fingerprint>>
-                                = HashMap::new();
+                            let mut i: BTreeMap<KeyID, Vec<Fingerprint>>
+                                = BTreeMap::new();
                             for c in certs.iter() {
                                 match i.entry(c.keyid()) {
                                     e @ Entry::Occupied(_) => {
@@ -802,15 +799,12 @@ Key ID ({}) matches multiple certificates: {}",
         store.precompute();
     }
 
-    let n = wot::Network::new(&store)?;
-
     let roots = wot::Roots::new(trust_roots.iter().cloned());
-    let mut q = wot::QueryBuilder::new(&n);
-    q.roots(roots);
+    let mut n = wot::NetworkBuilder::rooted(&store, roots);
     if cli.certification_network {
-        q.certification_network();
-    }
-    let mut q = q.build();
+        n = n.certification_network();
+    };
+    let mut n = n.build();
 
     let mut found_one = true;
     while found_one && ! possible_roots.is_empty() {
@@ -853,7 +847,7 @@ Key ID ({}) matches multiple certificates: {}",
                 }
 
                 let authenticated_amount
-                    = q.authenticate(
+                    = n.authenticate(
                         u.userid(), other_root.fingerprint(),
                         wot::FULLY_TRUSTED)
                     .amount();
@@ -869,7 +863,11 @@ Key ID ({}) matches multiple certificates: {}",
 
                     trust_roots.push(other_root);
                     let roots = wot::Roots::new(trust_roots.clone());
-                    q = wot::Query::new(&n, roots);
+                    let mut builder = wot::NetworkBuilder::rooted(&store, roots);
+                    if cli.certification_network {
+                        builder = builder.certification_network();
+                    };
+                    n = builder.build();
 
                     continue 'root;
                 } else {
@@ -890,24 +888,24 @@ Key ID ({}) matches multiple certificates: {}",
     match &cli.subcommand {
         cli::Subcommand::Authenticate { cert, userid, .. } => {
             // Authenticate a given binding.
-            authenticate(&cli, &q, cli.gossip, Some(userid), Some(cert))?;
+            authenticate(&cli, &n, cli.gossip, Some(userid), Some(cert))?;
         }
         cli::Subcommand::Lookup { userid, .. } => {
             // Find all authenticated bindings for a given
             // User ID, list the certificates.
-            authenticate(&cli, &q, cli.gossip, Some(userid), None)?;
+            authenticate(&cli, &n, cli.gossip, Some(userid), None)?;
         }
         cli::Subcommand::Identify { cert, .. } => {
             // Find and list all authenticated bindings for a given
             // certificate.
-            authenticate(&cli, &q, cli.gossip, None, Some(cert))?;
+            authenticate(&cli, &n, cli.gossip, None, Some(cert))?;
         }
         cli::Subcommand::List { .. } => {
             // List all authenticated bindings.
-            authenticate(&cli, &q, cli.gossip, None, None)?;
+            authenticate(&cli, &n, cli.gossip, None, None)?;
         }
         cli::Subcommand::Path { .. } => {
-            check_path(&cli, &q, policy)?;
+            check_path(&cli, &n, policy)?;
         }
     }
 
