@@ -3,8 +3,9 @@
 //! A [`Network`] accesses certificates via the [`Store`] interface.
 //!
 //! [`Network`]: crate::Network
-use std::collections::HashMap;
-use std::collections::HashSet;
+use std::collections::BTreeMap;
+use std::collections::BTreeSet;
+use std::sync::Arc;
 use std::time::SystemTime;
 
 use sequoia_openpgp as openpgp;
@@ -59,7 +60,7 @@ pub trait Backend<'a>: sequoia_cert_store::Store<'a> {
     /// `min_depth`.  If an implementation returns certifications
     /// whose trust depth is less than `min_depth` the caller must
     /// ignore them.
-    fn redges(&self, target: ValidCert, min_depth: Depth) -> Vec<CertificationSet>
+    fn redges(&self, target: ValidCert, min_depth: Depth) -> Arc<Vec<CertificationSet>>
     {
         tracer!(TRACE, "Backend::redges");
         t!("({}, {}; {})",
@@ -79,8 +80,8 @@ pub trait Backend<'a>: sequoia_cert_store::Store<'a> {
                           min_depth: Depth,
                           // If some, only return certifications by these
                           // issuers.
-                          filter: Option<HashSet<KeyID>>|
-            -> (Vec<Certification>, HashSet<KeyID>)
+                          filter: Option<BTreeSet<KeyID>>|
+            -> (Vec<Certification>, BTreeSet<KeyID>)
         {
             t!("Looking for active signatures: min depth: {}", min_depth);
 
@@ -88,16 +89,16 @@ pub trait Backend<'a>: sequoia_cert_store::Store<'a> {
 
             // If we've already seen a valid certification from the
             // Issuer on the current <Cert, UserID> binding.
-            let mut seen: HashMap<Fingerprint, std::time::SystemTime>
-                = HashMap::new();
+            let mut seen: BTreeMap<Fingerprint, std::time::SystemTime>
+                = BTreeMap::new();
 
             // If we skipped a certification due to the min_depth
             // constraint, and then find another certification that
             // does pass the constraint, we need to recheck the first
             // one, as it would be preferred.  We do this in a second
             // pass.
-            let mut skipped: HashSet<KeyID> = HashSet::new();
-            let mut retry: HashSet<KeyID> = HashSet::new();
+            let mut skipped: BTreeSet<KeyID> = BTreeSet::new();
+            let mut retry: BTreeSet<KeyID> = BTreeSet::new();
 
             'cert: for certification in certifications {
                 // Check that the certification is valid:
@@ -115,7 +116,63 @@ pub trait Backend<'a>: sequoia_cert_store::Store<'a> {
                         continue;
                     };
 
-                let alleged_issuers = certification.get_issuers();
+                let mut alleged_issuers = certification.get_issuers();
+
+                // Dedup the issuers as follows: if we have a
+                // fingerprint, and its key ID, looking it up by key
+                // ID will return the certificate.  So there is no
+                // need to look it up once by key ID and once by
+                // fingerprint.
+                if alleged_issuers.iter()
+                           .any(|i| {
+                               matches!(i, KeyHandle::Fingerprint(_))
+                           })
+                    && alleged_issuers.iter()
+                           .any(|i| {
+                               matches!(i, KeyHandle::KeyID(_))
+                           })
+                {
+                    t!("Dedupping {} alleged issuers: {}",
+                       alleged_issuers.len(),
+                       alleged_issuers
+                           .iter()
+                           .map(|i| format!("{}", i))
+                           .collect::<Vec<_>>()
+                           .join(", "));
+
+                    // Collect the key ids.
+                    let mut keyids = Vec::with_capacity(
+                        alleged_issuers.len() - 1);
+                    for issuer in alleged_issuers.iter() {
+                        if let KeyHandle::KeyID(keyid) = issuer {
+                            keyids.push(keyid.clone())
+                        }
+                    }
+
+                    // Get a list of fingerprints that are aliased.
+                    let mut kill = Vec::with_capacity(
+                        alleged_issuers.len() - keyids.len());
+                    for (i, issuer) in alleged_issuers.iter().enumerate() {
+                        if let KeyHandle::Fingerprint(fingerprint) = issuer {
+                            if keyids.contains(&KeyID::from(fingerprint)) {
+                                kill.push(i);
+                            }
+                        }
+                    }
+
+                    // Remove the aliased fingerprints.
+                    for i in kill.into_iter().rev() {
+                        alleged_issuers.swap_remove(i);
+                    }
+
+                    t!("Have {} alleged issuers after dedupping: {}",
+                       alleged_issuers.len(),
+                       alleged_issuers
+                           .iter()
+                           .map(|i| format!("{}", i))
+                           .collect::<Vec<_>>()
+                           .join(", "));
+                }
 
                 if min_depth > 0.into() {
                     let depth = certification.trust_signature()
@@ -334,8 +391,8 @@ pub trait Backend<'a>: sequoia_cert_store::Store<'a> {
 
         t!("Merging certifications.");
 
-        CertificationSet::from_certifications(
-            valid_certifications, reference_time)
+        Arc::new(CertificationSet::from_certifications(
+            valid_certifications, reference_time))
     }
 
     /// Prefills the cache.
@@ -356,7 +413,8 @@ pub trait Backend<'a>: sequoia_cert_store::Store<'a> {
 impl<'a: 't, 't, T> Backend<'a> for Box<T>
 where T: Backend<'a> + ?Sized + 't
 {
-    fn redges(&self, target: ValidCert, min_depth: Depth) -> Vec<CertificationSet>
+    fn redges(&self, target: ValidCert, min_depth: Depth)
+        -> Arc<Vec<CertificationSet>>
     {
         self.as_ref().redges(target, min_depth)
     }
@@ -369,7 +427,8 @@ where T: Backend<'a> + ?Sized + 't
 impl<'a: 't, 't, T> Backend<'a> for &'t T
 where T: Backend<'a> + ?Sized
 {
-    fn redges(&self, target: ValidCert, min_depth: Depth) -> Vec<CertificationSet>
+    fn redges(&self, target: ValidCert, min_depth: Depth)
+        -> Arc<Vec<CertificationSet>>
     {
         (*self).redges(target, min_depth)
     }
@@ -443,27 +502,21 @@ pub trait Store {
     /// whose trust depth is less than `min_depth` the caller must
     /// ignore them.
     fn certifications_of(&self, target: &Fingerprint, min_depth: Depth)
-        -> Result<Vec<CertificationSet>>;
+        -> Result<Arc<Vec<CertificationSet>>>;
 
     /// Returns all third-party certifications of the specified
     /// certificate.
-    ///
-    /// Returns [`StoreError::NotFound`] if the certificate is not
-    /// found.  This function SHOULD NOT return an empty vector if the
-    /// certificate is not found.
     fn third_party_certifications_of(&self, fpr: &Fingerprint)
         -> Vec<Certification>
     {
         self.certifications_of(fpr, 0.into())
-            .unwrap_or(Vec::new())
-            .into_iter().flat_map(|cs| {
+            .unwrap_or(Arc::new(Vec::new()))
+            .iter()
+            .flat_map(|cs| {
                 cs.certifications()
                     .flat_map(|(_userid, certifications)| {
-                        certifications
+                        certifications.iter().cloned()
                     })
-                    .cloned()
-                    .collect::<Vec<_>>()
-                    .into_iter()
             })
             .collect::<Vec<_>>()
     }
@@ -474,10 +527,6 @@ pub trait Store {
     /// This returns both self-signed User IDs, and User IDs certified
     /// by third-parties for the specified certificate.  The result is
     /// deduped.
-    ///
-    /// Returns [`StoreError::NotFound`] if the certificate is not
-    /// found.  This function SHOULD NOT return an empty vector if the
-    /// certificate is not found.
     fn certified_userids_of(&self, fpr: &Fingerprint)
         -> Vec<UserID>
     {
@@ -607,7 +656,7 @@ where T: Store + ?Sized + 't
     }
 
     fn certifications_of(&self, target: &Fingerprint, min_depth: Depth)
-        -> Result<Vec<CertificationSet>>
+        -> Result<Arc<Vec<CertificationSet>>>
     {
         self.as_ref().certifications_of(target, min_depth)
     }
@@ -668,7 +717,7 @@ where T: Store + ?Sized
     }
 
     fn certifications_of(&self, target: &Fingerprint, min_depth: Depth)
-        -> Result<Vec<CertificationSet>>
+        -> Result<Arc<Vec<CertificationSet>>>
     {
         (*self).certifications_of(target, min_depth)
     }
@@ -729,8 +778,7 @@ mod tests {
     use openpgp::policy::StandardPolicy;
 
     use crate::Network;
-    use crate::Query;
-    use crate::QueryBuilder;
+    use crate::NetworkBuilder;
 
     #[test]
     #[allow(unused)]
@@ -777,17 +825,17 @@ mod tests {
 
         // On the 14th, Bob should be a trusted introducer.
         t!("Jan 14, Bob");
-        let n = Network::from_raw_certs(certs.iter().cloned(), p, Some(jan14))?;
-        let r = Query::new(&n, &[ alice_fpr.clone() ]);
-        let got = r.authenticate(bob_uid.clone(),
+        let n = Network::from_raw_certs(certs.iter().cloned(), p, Some(jan14),
+                                        &[ alice_fpr.clone() ])?;
+        let got = n.authenticate(bob_uid.clone(),
                                  bob_fpr.clone(),
                                  120);
         assert!(got.amount() == 120);
 
         t!("Jan 14, Carol");
-        let n = Network::from_raw_certs(certs.iter().cloned(), p, Some(jan14))?;
-        let r = Query::new(&n, &[ alice_fpr.clone() ]);
-        let got = r.authenticate(carol_uid.clone(),
+        let n = Network::from_raw_certs(certs.iter().cloned(), p, Some(jan14),
+                                        &[ alice_fpr.clone() ])?;
+        let got = n.authenticate(carol_uid.clone(),
                                  carol_fpr.clone(),
                                  120);
         assert!(got.amount() == 120);
@@ -795,18 +843,18 @@ mod tests {
 
         // On the 16th, Bob should be a trusted introducer.
         t!("Jan 16, Bob");
-        let n = Network::from_raw_certs(certs.iter().cloned(), p, jan16)?;
-        let r = Query::new(&n, &[ alice_fpr.clone() ]);
+        let n = Network::from_raw_certs(certs.iter().cloned(), p, jan16,
+                                        &[ alice_fpr.clone() ])?;
 
-        let got = r.authenticate(bob_uid.clone(),
+        let got = n.authenticate(bob_uid.clone(),
                                  bob_fpr.clone(),
                                  120);
         assert!(got.amount() == 120);
 
         t!("Jan 16, Carol");
-        let n = Network::from_raw_certs(certs.iter().cloned(), p, jan16)?;
-        let r = Query::new(&n, &[ alice_fpr.clone() ]);
-        let got = r.authenticate(carol_uid.clone(),
+        let n = Network::from_raw_certs(certs.iter().cloned(), p, jan16,
+                                        &[ alice_fpr.clone() ])?;
+        let got = n.authenticate(carol_uid.clone(),
                                  carol_fpr.clone(),
                                  120);
         assert!(got.amount() == 0);
@@ -891,9 +939,7 @@ mod tests {
         // Do something (anything) with the backend.
         assert_eq!(foo.count(), 0);
 
-        let n = Network::new(backend).expect("valid");
-        let q = QueryBuilder::new(&n);
-        let _q = q.build();
+        let _n = NetworkBuilder::rootless(backend).build();
 
         Ok(())
     }
@@ -941,14 +987,13 @@ mod tests {
         assert_eq!(foo.count(), 0);
         assert_eq!(foo.count_synopses(), 0);
 
-        let n = Network::new(backend).expect("valid");
-        let mut q = QueryBuilder::new(&n);
-        q.certification_network();
-        let q = q.build();
+        let n = NetworkBuilder::rootless(backend)
+            .certification_network()
+            .build();
 
-        // Do something with q (anything).
+        // Do something with n (anything).
         eprintln!("{:?}",
-                  q.authenticate(
+                  n.authenticate(
                       UserID::from("<alice@example.org>"),
                       "0123 4567 89AB CDEF 0123 4567 89AB CDEF"
                           .parse::<Fingerprint>().expect("valid"),
