@@ -1,6 +1,6 @@
 use std::borrow::Borrow;
-use std::collections::HashMap;
-use std::collections::HashSet;
+use std::collections::BTreeMap;
+use std::collections::BTreeSet;
 use std::sync::{Arc, Mutex};
 use std::time::SystemTime;
 
@@ -85,7 +85,6 @@ use crate as wot;
 ///
 /// use sequoia_wot as wot;
 /// use wot::Network;
-/// use wot::Query;
 /// use wot::Roots;
 ///
 /// const P: &StandardPolicy = &StandardPolicy::new();
@@ -105,11 +104,10 @@ use crate as wot;
 ///     (alice.fingerprint().into(), wot::FULLY_TRUSTED),
 /// ]);
 /// let wot_data = wot::store::CertStore::from_store(&cert_store, P, None);
-/// let network = Network::new(&wot_data)?;
-/// let q = Query::new(&network, trust_roots.clone());
+/// let network = Network::new(&wot_data, trust_roots)?;
 ///
 /// // Try and authenticate Bob.
-/// let paths = q.authenticate(
+/// let paths = network.authenticate(
 ///     UserID::from("<bob@example.org>"),
 ///     bob.fingerprint(),
 ///     wot::FULLY_TRUSTED);
@@ -142,7 +140,7 @@ pub struct CertStore<'a: 'ra, 'ra, S>
     //
     // This is a cache, which is derived from the certs, and is update
     // lazily.
-    redge_cache: Mutex<HashMap<Fingerprint, Vec<CertificationSet>>>,
+    redge_cache: Mutex<BTreeMap<Fingerprint, Arc<Vec<CertificationSet>>>>,
 
     // The policy.  This is needed to compute the
     // certification sets.
@@ -466,10 +464,10 @@ impl<'a: 'ra, 'ra, S> Backend<'a> for CertStore<'a, 'ra, S>
         t!("prefetching");
 
         // Figure out what certificates we need to work on.
-        let all: HashSet<Fingerprint>
-            = HashSet::from_iter(self.store.fingerprints());
-        let done: HashSet<Fingerprint>
-            = HashSet::from_iter(
+        let all: BTreeSet<Fingerprint>
+            = BTreeSet::from_iter(self.store.fingerprints());
+        let done: BTreeSet<Fingerprint>
+            = BTreeSet::from_iter(
                 self.redge_cache.lock().unwrap().keys().cloned());
         let todo: Vec<&Fingerprint>
             = all.difference(&done).collect();
@@ -516,7 +514,7 @@ impl<'a: 'ra, 'ra, S> Backend<'a> for CertStore<'a, 'ra, S>
                 let reference_time = self.reference_time.clone();
 
                 threads_extant.push(thread_scope.spawn(move |_| {
-                    let mut results: Vec<(Fingerprint, Vec<CertificationSet>)>
+                    let mut results: Vec<(Fingerprint, Arc<Vec<CertificationSet>>)>
                         = Vec::new();
 
                     loop {
@@ -549,7 +547,8 @@ impl<'a: 'ra, 'ra, S> Backend<'a> for CertStore<'a, 'ra, S>
                                             the current policy: {}",
                                            cert.fingerprint(), err);
                                         results.push((
-                                            cert.fingerprint(), Vec::new()))
+                                            cert.fingerprint(),
+                                            Arc::new(Vec::new())))
                                     }
                                 }
                             }
@@ -569,7 +568,7 @@ impl<'a: 'ra, 'ra, S> Backend<'a> for CertStore<'a, 'ra, S>
         drop(work_tx);
 
         let redges = threads_extant.into_iter().flat_map(|t| {
-            let redges: Vec<(Fingerprint, Vec<CertificationSet>)>
+            let redges: Vec<(Fingerprint, Arc<Vec<CertificationSet>>)>
                 = t.join().unwrap();
             redges
         });
@@ -589,7 +588,7 @@ impl<'a: 'ra, 'ra, S> CertStore<'a, 'ra, S>
     where S: cert_store::store::Store<'a> + Send + Sync,
 {
     fn certifications_of_uncached<F>(&self, target: F)
-        -> Result<Vec<CertificationSet>>
+        -> Result<Arc<Vec<CertificationSet>>>
     where F: Borrow<Fingerprint>
     {
         let target = target.borrow();
@@ -659,7 +658,7 @@ impl<'a: 'ra, 'ra, S> Store for CertStore<'a, 'ra, S>
     }
 
     fn certifications_of(&self, target: &Fingerprint, _min_depth: Depth)
-        -> Result<Vec<CertificationSet>>
+        -> Result<Arc<Vec<CertificationSet>>>
     {
         tracer!(TRACE, "CertStore::certifications_of");
 
@@ -678,11 +677,36 @@ impl<'a: 'ra, 'ra, S> Store for CertStore<'a, 'ra, S>
 
         self.redge_cache
             .lock().unwrap()
-            .insert(target.clone(), redges.clone());
+            .insert(target.clone(), Arc::clone(&redges));
 
         Ok(redges)
     }
 
+    fn certified_userids_of(&self, fpr: &Fingerprint)
+        -> Vec<UserID>
+    {
+        let Ok(cert) = self.lookup_by_cert_fpr(fpr)
+        else { return Vec::new() };
+
+        cert.userids().collect()
+    }
+
+    fn certified_userids(&self)
+        -> Vec<(Fingerprint, UserID)>
+    {
+        self.certs()
+            .flat_map(|c| {
+                let fpr = c.fingerprint();
+                c.userids().into_iter()
+                    .map(|u| {
+                        (fpr.clone(), u)
+                    })
+                    .collect::<Vec<_>>()
+                    .into_iter()
+            })
+            .collect()
+    }
+
     fn lookup_synopses_by_userid(&self, userid: UserID) -> Vec<Fingerprint> {
         self.lookup_by_userid(&userid)
             .unwrap_or(Vec::new())
@@ -736,8 +760,7 @@ mod test {
     use cert_store::Store;
     use sequoia_cert_store::StoreUpdate;
 
-    use crate::Network;
-    use crate::Query;
+    use crate::NetworkBuilder;
     use crate::Roots;
     use crate::FULLY_TRUSTED;
 
@@ -748,8 +771,8 @@ mod test {
     fn cert_store_lifetimes() -> Result<()> {
         fn authenticate<'store: 'ra, 'ra>(
             // The core of our check.  Does the following compile:
-            n: &Network<CertStore<'store, 'ra,
-                                  &'ra cert_store::CertStore<'store>>>,
+            store: &CertStore<'store, 'ra,
+                              &'ra cert_store::CertStore<'store>>,
             trust_root: Fingerprint,
             target_fpr: Fingerprint,
             target_userid: UserID)
@@ -758,11 +781,13 @@ mod test {
             eprintln!("trust root: {}", trust_root);
             eprintln!("target: {}, {:?}", target_fpr, target_userid);
 
-            let q = Query::new(&n, Roots::from(&[
-                (trust_root, FULLY_TRUSTED),
-            ]));
+            let n = NetworkBuilder::rooted(
+                &store, Roots::from(&[
+                    (trust_root, FULLY_TRUSTED),
+                ]))
+                .build();
 
-            let paths = q.authenticate(
+            let paths = n.authenticate(
                 target_userid,
                 target_fpr,
                 FULLY_TRUSTED);
@@ -793,12 +818,11 @@ mod test {
 
         // Build a few WoT networks.
         for _ in 0..2 {
-            let wot_data = CertStore::from_store(&cert_store, P, None);
-            let network = Network::new(wot_data)?;
+            let store = CertStore::from_store(&cert_store, P, None);
 
             // Try and authenticate Bob.
             let amount = authenticate(
-                &network,
+                &store,
                 alice.fingerprint(),
                 bob.fingerprint(),
                 UserID::from("<bob@example.org>"));
@@ -809,7 +833,7 @@ mod test {
 
 
             let amount = authenticate(
-                &network,
+                &store,
                 alice.fingerprint(),
                 alice.fingerprint(),
                 UserID::from("<alice@example.org>"));
